mod\_authn\_dbd

User authentication using an SQL database

Extension

mod\_authn\_dbd.c

authn\_dbd\_module

This module provides authentication front-ends such as

mod\_auth\_digest

and

mod\_auth\_basic

to authenticate users by looking up users in SQL tables. Similar
functionality is provided by, for example,

mod\_authn\_file

.

This module relies on

mod\_dbd

to specify the backend database driver and connection parameters, and
manage the database connections.

When using

mod\_auth\_basic

or

mod\_auth\_digest

, this module is invoked via the

AuthBasicProvider

or

AuthDigestProvider

with the

dbd

value.

AuthName

AuthType

AuthBasicProvider

AuthDigestProvider

DBDriver

DBDParams

Password Formats

Performance and Caching {#socache}
=======================

Some users of DBD authentication in HTTPD 2.2/2.4 have reported that it
imposes a problematic load on the database. This is most likely where an
HTML page contains hundreds of objects (e.g. images, scripts, etc) each
of which requires authentication. Users affected (or concerned) by this
kind of problem should use

mod\_authn\_socache

to cache credentials and take most of the load off the database.

Configuration Example {#example}
=====================

This simple example shows use of this module in the context of the
Authentication and DBD frameworks.

\# mod\_dbd configuration \# UPDATED to include authentication caching
DBDriver pgsql DBDParams \"dbname=apacheauth user=apache
password=xxxxxx\" DBDMin 4 DBDKeep 8 DBDMax 20 DBDExptime 300
\<Directory \"/usr/www/myhost/private\"\> \# mod\_authn\_core and
mod\_auth\_basic configuration \# for mod\_authn\_dbd AuthType Basic
AuthName \"My Server\" \# To cache credentials, put socache ahead of dbd
here AuthBasicProvider socache dbd \# Also required for caching: tell
the cache to cache dbd lookups! AuthnCacheProvideFor dbd
AuthnCacheContext my-server \# mod\_authz\_core configuration Require
valid-user \# mod\_authn\_dbd SQL query to authenticate a user
AuthDBDUserPWQuery \"SELECT password FROM authn WHERE user = %s\"
\</Directory\>

Exposing Login Information {#exposed}
==========================

If httpd was built against

APR

version 1.3.0 or higher, then whenever a query is made to the database
server, all column values in the first row returned by the query are
placed in the environment, using environment variables with the prefix
\"AUTHENTICATE\_\".

If a database query for example returned the username, full name and
telephone number of a user, a CGI program will have access to this
information without the need to make a second independent database query
to gather this additional information.

This has the potential to dramatically simplify the coding and
configuration required in some web applications.

Preventing SQL injections {#security}
=========================

Whether you need to care about SQL security depends on what DBD driver
and backend you use. With most drivers you don\'t have to do anything :
the statement is prepared by the database at startup, and user input is
used only as data. But you may need to untaint your input. At the time
of writing, the only driver that requires you to take care is FreeTDS.

Please read

mod\_dbd

documentation for more information about security on this scope.

AuthDBDUserPWQuery

SQL query to look up a password for a user

AuthDBDUserPWQuery

query

directory

The

AuthDBDUserPWQuery

specifies an SQL query to look up a password for a specified user. The
user\'s ID will be passed as a single string parameter when the SQL
query is executed. It may be referenced within the query statement using
a

\%s

format specifier.

AuthDBDUserPWQuery \"SELECT password FROM authn WHERE user = %s\"

The first column value of the first row returned by the query statement
should be a string containing the encrypted password. Subsequent rows
will be ignored. If no rows are returned, the user will not be
authenticated through

mod\_authn\_dbd

.

If httpd was built against

APR

version 1.3.0 or higher, any additional column values in the first row
returned by the query statement will be stored as environment variables
with names of the form

AUTHENTICATE\_

COLUMN

.

The encrypted password format depends on which authentication frontend
(e.g.

mod\_auth\_basic

or

mod\_auth\_digest

) is being used. See

Password Formats

for more information.

AuthDBDUserRealmQuery

SQL query to look up a password hash for a user and realm.

AuthDBDUserRealmQuery

query

directory

The

AuthDBDUserRealmQuery

specifies an SQL query to look up a password for a specified user and
realm in a digest authentication process. The user\'s ID and the realm,
in that order, will be passed as string parameters when the SQL query is
executed. They may be referenced within the query statement using

\%s

format specifiers.

AuthDBDUserRealmQuery \"SELECT password FROM authn WHERE user = %s AND
realm = %s\"

The first column value of the first row returned by the query statement
should be a string containing the encrypted password. Subsequent rows
will be ignored. If no rows are returned, the user will not be
authenticated through

mod\_authn\_dbd

.

If httpd was built against

APR

version 1.3.0 or higher, any additional column values in the first row
returned by the query statement will be stored as environment variables
with names of the form

AUTHENTICATE\_

COLUMN

.

The encrypted password format depends on which authentication frontend
(e.g.

mod\_auth\_basic

or

mod\_auth\_digest

) is being used. See

Password Formats

for more information.
